
# Supplementary material to the article
# "Contingent Valuation: 
# how opportunity costs influence the stated willingness to pay"
# by Ulrich B. Morawetz und Dieter B. Koemle
# The code is written in the software R
# which is freely available on www.r-project.org

# The code consists of three parts
# Part 1) Table A1 and Figure A1 is generated
# Part 2) the code used for the simulations in the Appendix of the paper including generation of Figure A2, Figure A3 and Table A2
# Part 3) explains how the data of Bishop et al. (2017) have been analyzed and generates Table 3 of the main text

## set your working directory where you saved the csv files 
#setwd("E:/mydirectory")


 # select in which format figures should be saved

  #FigureFormat<-"tiff"
   FigureFormat<-"png"   
  #FigureFormat<-"eps"
 

#############################
# Part 1
# Table A1 and Figure A1 is generated   
#############################
 
 #### keep t' and t'' fixed and use two levels of and t0' 
 # show how the stated WTP changes with different t0'' 

  surplusOffered <- c(10, 10,  10, -10, -10,-10) #needs to be according to the utility ranking of the six cases
  surplusAlterna <- c( 5, 25, -10,  25, -15, -5) #needs to be according to the utility ranking of the six cases
  tprime         <- c(20, 20,  20,  20,  20, 20)
  tnodprime      <- tprime - surplusOffered       
  tprimeprime    <- c(25, 25, 25, 25, 25, 25)+25  
  tnodprimeprime <- tprimeprime - surplusAlterna

 

  points<-cbind(tprime, tnodprime, tprimeprime, tnodprimeprime)
  statedWTP<- tprime-(tprimeprime-tnodprimeprime)
  
 example<-cbind(tprime, tnodprime, surplusOffered, tprimeprime, tnodprimeprime, surplusAlterna,sign(surplusOffered-surplusAlterna) , statedWTP)
  rownames(example) <-  c("B1","B2","B3", "B4", "B5", "B6")
  colnames(example) <- c("t'", "t0'", "Surplus offered public good", "t''", "t0''", "Surplus alternative public good", "Vote", "min/max t0' to change vote") 

 write.csv(example,file = "Table_A1.csv")



 if(FigureFormat=="tiff")
  {tiff(filename = "Figure_A1.tiff",  width = 900, height = 450)
   par(cex=1.5)
   }
 if(FigureFormat=="eps")
  {postscript(file="Figure_A1.eps", paper="special", onefile=FALSE, horizontal=FALSE,  width=10,    height=4.2 ) }
 if(FigureFormat=="png")
  {png(filename = "Figure_A1.png",   width = 90*.35, height = 45*.35, res=600, units="cm")
   par(cex=1.5)
   }              

 plot(x=c(min(0,-20), max(points)*1.3), y=c(0,6.5), type="n", 
     xlab="", ylab="", yaxt="n", xaxt="n", bty="n", omi=c())

 ticksAt <- c(-5, 0, 5, 10,15, 20, 25, 30,35, 40,45, 50, 55, 60, 65)
  axis(1, tick=TRUE, lwd=0, las=1, 
     at=ticksAt )
  axis(2, at=1:6, labels= c("B6","B5","B4","B3","B2","B1"),
     tick=FALSE, lwd=0, las=1 )

 abline(v=0)
 abline(h=0)
 
 j<-0
  for(i in 6:1){
    j<-j+1
 arrows(x0=min(0,-20),x1=statedWTP[i], y0=j, y1=j, col="black", lwd=3)
 arrows(x0=max(points)*1.3           ,x1=statedWTP[i], y0=j, y1=j, col="gray", lwd=3)
 points( x=c(tnodprime[i], tnodprimeprime[i], tprime[i], tprimeprime[i]), y=c(j,j,j, j) , pch=19)
 text( x=c(tnodprime[i], tnodprimeprime[i], tprime[i],tprimeprime[i]), y=c(j,j, j, j), 
      labels=c(expression(paste(t[0],"'", sep="")),
               expression(paste(t[0],"''", sep="")), 
               "t'", 
               "t''") , pos=1)
 points(x=ticksAt , y=rep(0,times=length(ticksAt)), pch=3)
 text( x=c(min(0,-20*.95), max(points)*1.2), y=c(j,j), 
      labels=c("'yes'","'no'") , pos=c(1,1), col=c("black","black") )
 }

 par(cex=1)
 dev.off()

#############################
# Part 2 
# Simulations shown in the appendix and Figure A2, A3 and Table A2 are generated
############################

#############################
##### Simulation 
 
 # set the number of observations for the simulation 
  obs<-100000

 # generate the three data sets  
   set.seed(12345)
    sim1<-data.frame("tprime"         = rep(20, times=obs),
                     "tnodprime"      = sample(c(10,30), obs, replace=TRUE),
                     "surplAlt"       = rnorm(obs,mean=0, sd=25)
                   )
   set.seed(12345)
    sim2<-data.frame("tprime"       = rep(20, times=obs),
                     "tnodprime"      = sample(c(10,30), obs, replace=TRUE),
                     "surplAlt"       = rnorm(obs,mean=50, sd=25) 
                 )
   set.seed(12345)
    sim3<-data.frame("tprime"       = rep(20, times=obs),
                     "tnodprime"      = sample(c(10,30), obs, replace=TRUE),
                     "surplAlt"       = rnorm(obs,mean=-10, sd=25) 
                 )
  ######################
  # Simulation under certainty
  
  # determine the maximum t0' to vote "yes"
   sim1$switch<-with(sim1, tprime-surplAlt)
   sim2$switch<-with(sim2, tprime-surplAlt)
   sim3$switch<-with(sim3, tprime-surplAlt)
  # set answer according to the decision rule: "yes" if t0'<t'-(t''-t0'')
   # add a random error as otherwise there is non-convergence when surplAlt is added as explanatory variable
    set.seed(12345); error<-rnorm(obs)
    sim1$yes   <- 0
    sim2$yes   <- 0
    sim3$yes   <- 0  
    sim1$yes[with(sim1, tnodprime<(switch+error))] <- 1
    sim2$yes[with(sim2, tnodprime<(switch+error))] <- 1
    sim3$yes[with(sim3, tnodprime<(switch+error))] <- 1

  
  # summary statistics of simulated data  
   # mean surplus alternative public good
    meanSurplusAltPubGood1 <- with(sim1, mean(surplAlt) )
    meanSurplusAltPubGood2 <- with(sim2, mean(surplAlt) )
    meanSurplusAltPubGood3 <- with(sim3, mean(surplAlt) )   
   # mean maximum level to vote "yes" 
    meanSwitchLevel1 <-mean(sim1$switch)
    meanSwitchLevel2 <-mean(sim2$switch)
    meanSwitchLevel3 <-mean(sim3$switch)   

  # Draw figure A2 
  if(FigureFormat=="tiff")   
   {tiff(filename = "Figure_A2.tiff",  width = 900, height = 450)  
   par(cex=1.5)
   }
  if(FigureFormat=="eps")  
   {postscript(file="Figure_A2.eps",   paper="special", onefile=FALSE, horizontal=FALSE,  width=8.4,    height=4.2) }
  if(FigureFormat=="png")   
   {png(filename = "Figure_A2.png",   width = 90*.35, height = 45*.35, res=600, units="cm")  
   par(cex=1.5)
   }
   
  # Distribution of maximum level to vote "yes"
   plot(range( density(sim1$switch)$x, density(sim2$switch)$x, density(sim3$switch)$x), 
        range( density(sim1$switch)$y, density(sim2$switch)$y,density(sim3$switch)$y), 
        type = "n", xlab = "", ylab = "Density",
       xlim=c(-100,100) )
    lines(density(sim1$switch), col = "black", lwd=2, lty=1)
    lines(density(sim2$switch), col = "black",   lwd=3, lty=2)
    lines(density(sim3$switch), col = "black",  lwd=3, lty=3)
    legend("topleft", legend=c( 
                              expression(paste("t'-(t''-", t[0] , "'') for Set 1", sep="")),
                              expression(paste("t'-(t''-", t[0] , "'') for Set 2", sep="")),
                              expression(paste("t'-(t''-", t[0] , "'') for Set 3", sep="")),
                              "true WTP"
                              ),
          lwd=c(2,3,3,3), lty=c(1,2,3,1),
          bty="n", col=c("black","black", "black","darkgray") )
    abline(v=c(20), lwd=3, col="darkgray")
    abline(v=0)

   axis(1, tick=TRUE,  las=1, at=c(-30,20,30) )
   par(cex=1)  
  dev.off()
  
 # Table A2 (upper two panels)
 
  ### run binary probit regressions 
  sb1 <-  glm(formula=yes~1+tnodprime, family =  binomial(link = "probit"), data=sim1)
  sb2 <-  glm(formula=yes~1+tnodprime, family =  binomial(link = "probit"), data=sim2)
  sb3 <-  glm(formula=yes~1+tnodprime, family =  binomial(link = "probit"), data=sim3)
 
  simSum  <- cbind(
          "meanValueOfferdGood"=c(mean(sim1$tprime), mean(sim2$tprime),mean(sim3$tprime)),
          "meanContrOfferdGood"=c(mean(sim1$tnodprime), mean(sim2$tnodprime),mean(sim3$tnodprime)),
          "meanSurplusAltPubl"= c(meanSurplusAltPubGood1,meanSurplusAltPubGood2, meanSurplusAltPubGood3),
          "meanSwitchLevel" = c(meanSwitchLevel1, meanSwitchLevel2, meanSwitchLevel3 ),
          "meanYes" = c(mean(sim1$yes), mean(sim2$yes),mean(sim3$yes)),
          "meanWTP" = c (with(sb1, coefficients[1]/coefficients[2]),
                         with(sb2, coefficients[1]/coefficients[2]),
                         with(sb3, coefficients[1]/coefficients[2]))
         )
        rownames(simSum)<- c("Set 1", "Set 2", "Set 3")
  t(round(simSum, digits=2))
   write.csv(t(round(simSum, digits=2)),file = "Table_A2_UpperTwoPanels.csv")


 ## Adding the surplus of the alternative public good 
 # and thereby controlling for bias
  sb1.noBias <-  glm(formula=yes~1+tnodprime + surplAlt, family =  binomial(link = "probit"), data=sim1)
  sb2.noBias <-  glm(formula=yes~1+tnodprime + surplAlt, family =  binomial(link = "probit"), data=sim2)
  sb3.noBias <-  glm(formula=yes~1+tnodprime + surplAlt, family =  binomial(link = "probit"), data=sim3)
  with(sb1.noBias, coefficients[1]/coefficients[2])
  with(sb2.noBias, coefficients[1]/coefficients[2])
  with(sb3.noBias, coefficients[1]/coefficients[2])


 ###############################
 ### Simulation under uncertainty

 # generate alpha:
 set.seed(12345)
 alpha<-runif(obs)
  
  # determine the maximum t0' to vote "yes"
   sim1$switchRisk<-with(sim1, tprime- alpha * surplAlt)
   sim2$switchRisk<-with(sim2, tprime- alpha * surplAlt)
   sim3$switchRisk<-with(sim3, tprime- alpha * surplAlt)
  # set answer according to the decision rule: "yes" if t0'< t'- alpha * (t''-t0'')
   sim1$yesRisk   <- 0
   sim2$yesRisk   <- 0
   sim3$yesRisk   <- 0  
   sim1$yesRisk[with(sim1, tnodprime<switchRisk+error)] <- 1
   sim2$yesRisk[with(sim2, tnodprime<switchRisk+error)] <- 1
   sim3$yesRisk[with(sim3, tnodprime<switchRisk+error)] <- 1

  # summary statistics of simulated data  
   # mean surplus alternative public good
    meanSurplusAltPubGoodRisk1 <- with(sim1, mean(surplAlt) )
    meanSurplusAltPubGoodRisk2 <- with(sim2, mean(surplAlt) )
    meanSurplusAltPubGoodRisk3 <- with(sim3, mean(surplAlt) )   
   # mean maxium level to vote "yes" 
    meanSwitchLevelRisk1 <-mean(sim1$switchRisk)
    meanSwitchLevelRisk2 <-mean(sim2$switchRisk)
    meanSwitchLevelRisk3 <-mean(sim3$switchRisk)   

  # Draw figure A3 
  if(FigureFormat=="tiff") 
   {tiff(filename = "Figure_A3.tiff", width = 900, height = 450)
   par(cex=1.5)
   }
  if(FigureFormat=="eps")
   { postscript(file="Figure_A3.eps",      paper="special", onefile=FALSE, horizontal=FALSE,    width=8.4,    height=4.2) }
  if(FigureFormat=="png") 
   {png(filename = "Figure_A3.png", width = 90*.35, height = 45*.35, res=600, units="cm")
   par(cex=1.5)
   }    
    
   
 # Distribution of maximum level to vote "yes"
  plot(range( density(sim1$switchRisk)$x, density(sim2$switchRisk)$x, density(sim3$switchRisk)$x), 
        range( density(sim1$switchRisk)$y, density(sim2$switchRisk)$y,density(sim3$switchRisk)$y), 
        type = "n", xlab = "", ylab = "Density",
       xlim=c(-100,100) )
   lines(density(sim1$switchRisk), col = "black",  lwd=2, lty=1)
   lines(density(sim2$switchRisk), col = "black",  lwd=3, lty=2)
   lines(density(sim3$switchRisk), col = "black",  lwd=3, lty=3)
   legend("topleft", legend=c(expression(paste("t'-", alpha,"(t''-", t[0] , "'') for Set 1", sep="")),
                              expression(paste("t'-", alpha,"(t''-", t[0] , "'') for Set 2", sep="")),
                              expression(paste("t'-", alpha,"(t''-", t[0] , "'') for Set 3", sep="")),
                              "true WTP"
                              ),
          lwd=c(2,3,3,3), lty=c(1,2,3,1),
          bty="n", col=c("black","black", "black","darkgray") )
   abline(v=c(20), lwd=3, col="darkgray")
   abline(v=0)
  axis(1, tick=TRUE,  las=1, at=c(-30,20,30) )
 
   par(cex=1)
   dev.off()
 

 # Table A2 (top and lowest panel)

  ### run binary probit regressions 
   sb1Risk <-  glm(formula=yesRisk~1+tnodprime, family =  binomial(link = "probit"), data=sim1)
   sb2Risk <-  glm(formula=yesRisk~1+tnodprime, family =  binomial(link = "probit"), data=sim2)
   sb3Risk <-  glm(formula=yesRisk~1+tnodprime, family =  binomial(link = "probit"), data=sim3)

   simRiskSum  <- cbind(
          "meanValueOfferdGood"=c(mean(sim1$tprime), mean(sim2$tprime),mean(sim3$tprime)),
          "meanContrOfferdGood"=c(mean(sim1$tnodprime), mean(sim2$tnodprime),mean(sim3$tnodprime)),
          "meanSurplusAltPubl"= c(meanSurplusAltPubGood1,meanSurplusAltPubGood2, meanSurplusAltPubGood3),
          "meanSwitchLevel" = c(meanSwitchLevelRisk1, meanSwitchLevelRisk2, meanSwitchLevelRisk3 ),
          "meanYes" = c(mean(sim1$yesRisk), mean(sim2$yesRisk),mean(sim3$yesRisk)),
          "meanWTP" = c (with(sb1Risk, coefficients[1]/coefficients[2]),
                         with(sb2Risk, coefficients[1]/coefficients[2]),
                         with(sb3Risk, coefficients[1]/coefficients[2]))
         )
        rownames(simRiskSum)<- c("Set 1", "Set 2", "Set 3")

  # The table in the appendix
   t(round(simRiskSum, digits=2))
 
   write.csv(t(round(simRiskSum, digits=2)),file = "Table_A2_UpperAndLowerPanel.csv")


 ## Adding a the surplus of the alternative public good 
 # and thereby controlling for bias 
  sb1Risk.noBias <-  glm(formula=yesRisk~1+tnodprime + I(alpha*surplAlt), family =  binomial(link = "probit"), data=sim1)
  sb2Risk.noBias <-  glm(formula=yesRisk~1+tnodprime + I(alpha*surplAlt), family =  binomial(link = "probit"), data=sim2)
  sb3Risk.noBias <-  glm(formula=yesRisk~1+tnodprime + I(alpha*surplAlt), family =  binomial(link = "probit"), data=sim3)

  with(sb1Risk.noBias, coefficients[1]/coefficients[2])
  with(sb2Risk.noBias, coefficients[1]/coefficients[2])
  with(sb3Risk.noBias, coefficients[1]/coefficients[2])


 
#############################
# Part 3 
# Explains how the data of Bishop et al. (2017) have been analyzed and generates Table 3 of the main text 
############################# 
 
#################################################
#######
# Instructions how the the data appendix of Bishop et al. (2017) have been analyzed
 ## Technical details are available from  "https://www.doi.gov/deepwaterhorizon/adminrecord"
 ## by typing "total value" in the search box
 
 
## instructions how to get data from Bishop et al. (2017) "Putting a value on injuries to natural assets: the BP oil spill" Science, Vol 356 Issue 6335 p. 253-254:
 ## the data is available from "https://www.diver.orr.noaa.gov/deepwater-horizon-nrda-data/"
 ## The relevant data are found by following the link "Download data package files ...". 
 ## The two data sets listed under "Total Value" are the relevant ones
 ## Download these data. Open them and save them as csv files under the names
 ## "Total Value Study Open Ended Response Codes.csv" and "Total Value Study Main and Non-Response Follow-Up Survey Data.csv"



# read the csv files
 openQuestion<-read.csv("Total Value Study Open Ended Response Codes.csv")
 closedQuestions<-read.csv("Total Value Study Main and Non-Response Follow-Up Survey Data.csv")

# merge the two csv files

 bis2017full<-merge(openQuestion, closedQuestions, by="id", all=TRUE)

 bis2017full$voteYes<- NA
 bis2017full$voteYes[bis2017full$q24==1]<- 1 # yes
 bis2017full$voteYes[bis2017full$q24==2]<- 0 # no
 
 
 # make a dataset consisting only of those who replied  
  bis2017r<-bis2017full[bis2017full$nrfu==0,]
 
  
 # Reasons for voting are given in the variables names starting with "code"
 # bis2017$code56 "Gov't should spend money on other programs beside oil spills, or the gov't shouldn't add new programs or shouldn't be involved or responsible"
 # For a list of codings see online "Appendix 4.1 Coding of Open-Ended Questions." from Bishop et al. (2017)
  

  # check the NAs
  table(is.na(bis2017r$voteYes))/sum(table(is.na(bis2017r$voteYes)))
  table(is.na(bis2017r$code56) )/sum(table(is.na(bis2017r$voteYes)))
  
  
  # Total votes  
  TotalNum<-with(bis2017r[!is.na(bis2017r$code56)&!is.na(bis2017r$voteYes),], sum(table(code56)))
  YNum    <-with(bis2017r[!is.na(bis2017r$code56)&!is.na(bis2017r$voteYes),], table(voteYes)[2] )
  NNum    <-with(bis2017r[!is.na(bis2017r$code56)&!is.na(bis2017r$voteYes),], table(voteYes)[1] )
  
  c(TotalNum,YNum, NNum)
    
  # share of votes with code 56
  code56_totalPerc<-with(bis2017r[!is.na(bis2017r$code56)&!is.na(bis2017r$voteYes),],   (table(code56)/TotalNum [1])[2]*100 )

  # share "yes" voters with with code 56 and share "no" voters with code 56
   code56_NPerc<- with(bis2017r[!is.na(bis2017r$code56)&!is.na(bis2017r$voteYes),], (table(voteYes, code56)[,2]/rowSums(table(voteYes, code56))[1])[1]*100 )
   code56_YPerc<- with(bis2017r[!is.na(bis2017r$code56)&!is.na(bis2017r$voteYes),], (table(voteYes, code56)[,2]/rowSums(table(voteYes, code56))[1])[2]*100 )
  

  # obsevation used for citation in manuscript 
  bis2017r[bis2017r$id==781,c("q35", "q36", "q49")]
  # q36 age: survey was done in 2013 and 2014
  # 2014-1951
  # q36 educatin: code 11: Professional degree
  # q49 gender: 2= female
  
  # List all code56 statements  
   write.csv(data.frame("id"=bis2017r$id[bis2017r$code56==1&!is.na(bis2017r$code56)], "replyCode56"=bis2017r$o26[bis2017r$code56==1&!is.na(bis2017r$code56)]),
              "code56replies.csv", row.names=FALSE )
   
  
  # counting manually all code56 answers which fit "Gov't should spend money on other programs beside oil spills"  

  # read the file with the manually categorized 56 sub-categories  
   code56repliesCategories<-read.csv("code56repliesCategories.csv")
   code56repliesCategories$replyCode56<-NULL
   #rename
   names(code56repliesCategories)<-c("id", "SpendOtherPrograms", "NoNewPrograms", "GovNotInvolvedResp") 
   # merge with other data
   bis2017rCode56<-merge(bis2017r, code56repliesCategories, by="id", all=TRUE)  
   
   
   # set to zero when NA (i.e. whenever not code56)
   bis2017rCode56$SpendOtherPrograms[is.na(bis2017rCode56$SpendOtherPrograms)]<-0
   bis2017rCode56$NoNewPrograms[is.na(bis2017rCode56$NoNewPrograms)]<-0
   bis2017rCode56$GovNotInvolvedResp[is.na(bis2017rCode56$GovNotInvolvedResp)]<-0
  

  # share of votes with code 56 "Gov't should spend money on other programs beside oil spills"
   code56sub_totalPerc<-with(bis2017rCode56[!is.na(bis2017rCode56$code56)&!is.na(bis2017rCode56$voteYes),],    
                             table(SpendOtherPrograms)[2] /TotalNum *100)
  
  # vote and code 56 subcategory "Gov't should spend money on other programs beside oil spills" 
    table(bis2017rCode56$voteYes, bis2017rCode56$SpendOtherPrograms)
  # share of "no voters among replies from code56 subcategory "Gov't should spend money on other programs beside oil spills"
   code56sub_NPerc<- with(bis2017rCode56[!is.na(bis2017rCode56$code56)&!is.na(bis2017rCode56$voteYes),],
                (table(voteYes, SpendOtherPrograms)[1,2] /NNum) ) *100
  # share "yes" voters with with code 56 "Gov't should spend money on other programs beside oil spills" from "yes" voters
   code56sub_YPerc<-with(bis2017rCode56[!is.na(bis2017rCode56$code56)&!is.na(bis2017rCode56$voteYes),],
                (table(voteYes, SpendOtherPrograms)[2,2] /YNum) ) *100
   
   
   DeepWaterTable<-rbind(
   c(TotalNum,YNum, NNum),
   c(code56_totalPerc, code56_YPerc, code56_NPerc),
   c(code56sub_totalPerc, code56sub_YPerc, code56sub_NPerc)
   )

   colnames(DeepWaterTable)<-c("All", "Yes-voters", "No-voters")
   rownames(DeepWaterTable)<-c("NumberRespondents", "Code56", "Code56sub")   
   
   round(DeepWaterTable,2)
   
 write.csv(DeepWaterTable, file= "Table_3.csv")   
     

###### References
#  Bishop, R.C., Boyle, K.J., Carson, R.T., Chapman, D., Hanemann, W.M., Kanninen, B., Kopp, R.J., Krosnick, J.A., List, J., Meade, N., Paterson, R., Presser, S., Smith, V.K., Tourangeau, R., Welsh, M., Wooldridge, J.M., DeBell, M., Donovan, C., Konopka, M., Scherer, N., 2017. Putting a value on injuries to natural assets: The BP oil spill. Science 356, 253–254. doi:10.1126/science.aam8124
  
